library CreateClientOrder;

uses
  Windows, SysUtils, Variants,
  ExaPlugin_TLB in 'ExaPlugin_TLB.pas';

{$R *.res}

type
  TPlugin = class(TInterfacedObject, IUnknown, IExaDataBaseCommandsPlugin)
  private
    FMainFormHandle: HWND;
    FDataBase: IExaDataBase;
    procedure CreateClient;
    procedure CreateOrder;
  protected
    function Get_CommandCount: Integer; stdcall;
    function Get_Caption(Command: Integer): WideString; stdcall;
    procedure Execute(Command: Integer; const ExaApp: IExaApp); stdcall;
  end;

{ TPlugin }

procedure TPlugin.CreateClient;
var
  LClients: IExaTable;
  LName, LClientID: string;
begin
  LClients := FDataBase.TableByName('Clients');
  if not LClients.UserHasWriteAccess then
    raise Exception.Create('    ');
  FDataBase.BeginTransaction;
  try
    LName := '  ';
    LClientID := LClients.Locate1('NAME', LName);
    if LClientID = '' then
    begin
      LClientID := LClients.Add;
      LClients.Set_Values(LClientID, 'NAME', LName); //  
    end;
    LClients.Set_Values(LClientID, 'BIRTHDAY', EncodeDate(1999, 03, 22));
    LClients.Set_Values(LClientID, 'PHONE', '+7(900)001-02-03');
    LClients.Set_Values(LClientID, 'EMAIL', 'ivanovii1999@bk.ru');
    LClients.Set_Values(LClientID, 'NOTES', '  ,' + sLineBreak + ' !');
  finally
    FDataBase.EndTransaction;
  end;
  MessageBox(FMainFormHandle, PChar(' : ' + LName), ' ', MB_OK);
end;

procedure TPlugin.CreateOrder;

  function NewRandomOrderKey: string;
  begin
    //    
    Result := (100000 + Random(900000)).ToString;
  end;

  function IDArrayToVariant(const IDArray: TArray<string>): Variant;
  var
    I: Integer;
  begin
    //      
    if Length(IDArray) > 0 then
    begin
      Result := VarArrayCreate([Low(IDArray), High(IDArray)], varOleStr);
      for I := Low(IDArray) to High(IDArray) do
        Result[I] := IDArray[I];
    end
    else
      Result := Null;
  end;

var
  LClients, LRooms, LOrders, LOrderLiving: IExaTable;
  LOrderID, LOrderLivingID, LClientID, LRoomID, LRoomCaption, LOrderKey: string;
  LClientsArr: TArray<string>;
begin
  LClients := FDataBase.TableByName('Clients');
  if not LClients.UserHasReadAccess then
    raise Exception.Create('    ');
  LRooms := FDataBase.TableByName('Objects');
  if not LRooms.UserHasReadAccess then
    raise Exception.Create('    ');
  LOrders := FDataBase.TableByName('Orders');
  LOrderLiving := FDataBase.TableByName('OrderLiving');
  if (not LOrders.UserHasWriteAccess) or (not LOrderLiving.UserHasWriteAccess) then
    raise Exception.Create('    ');

  FDataBase.BeginTransaction;
  try
    LRoomID := LRooms.First; //     
    LRoomCaption := LRooms.Get_Values(LRoomID, 'NAME'); //   

    //   
    LClientsArr := nil;
    LClientID := LClients.Locate1('NAME', '  '); //   
    if LClientID <> '' then
      LClientsArr := LClientsArr + [LClientID];
    LClientID := LClients.Locate1('NAME', '  '); //    
    if LClientID <> '' then
      LClientsArr := LClientsArr + [LClientID];

    //    
    Randomize;
    LOrderKey := NewRandomOrderKey;
    while LOrders.Locate1('ORDERKEY', LOrderKey) <> '' do
      LOrderKey := NewRandomOrderKey;

    //  
    LOrderID := LOrders.Add;
    LOrders.Set_Values(LOrderID, 'ORDERKEY', LOrderKey);
    LOrders.Set_Values(LOrderID, 'STATE', ''); //  ;       
    LOrders.Set_Values(LOrderID, 'DATEFROM', EncodeDate(2019, 05, 01));
    LOrders.Set_Values(LOrderID, 'TIMEFROM', EncodeTime(09, 00, 00, 00));
    LOrders.Set_Values(LOrderID, 'DATETO', EncodeDate(2019, 05, 14));
    LOrders.Set_Values(LOrderID, 'OBJECTIDS', LRoomID); //  :    ID     OleVariant
    LOrders.Set_Values(LOrderID, 'CLIENTIDS', IDArrayToVariant(LClientsArr)); //  :    ID     OleVariant
    LOrders.Set_Values(LOrderID, 'PLACES', Length(LClientsArr)); //  
    LOrders.Set_Values(LOrderID, 'PRICENUMBER', ' '); //  ;       
    LOrders.Set_Values(LOrderID, 'SUMM', 18500); //   
    LOrders.Set_Values(LOrderID, 'SUMMRECEIVED', 3000.50); //  
    LOrders.Set_Values(LOrderID, 'NOTES', '   9 ');

    //    
    LOrderLivingID := LOrderLiving.Add;
    LOrderLiving.Set_Values(LOrderLivingID, 'ORDERID', LOrderID); //   
    LOrderLiving.Set_Values(LOrderLivingID, 'CUSTOMNAME', '   ' + LRoomCaption);
    LOrderLiving.Set_Values(LOrderLivingID, 'COUNT', 1);
    LOrderLiving.Set_Values(LOrderLivingID, 'PRICE', 18500);
  finally
    FDataBase.EndTransaction;
  end;
  MessageBox(FMainFormHandle, PChar(' : ' + LOrderKey), ' ', MB_OK);
end;

procedure TPlugin.Execute(Command: Integer; const ExaApp: IExaApp);
begin
  if ExaApp.Get_Name <> 'ExaHotel' then
    MessageBox(ExaApp.Get_MainFormHandle, '     ExaHotel',
      PChar(Get_Caption(Command)), MB_OK)
  else begin
    try
      FMainFormHandle := ExaApp.Get_MainFormHandle;
      FDataBase := ExaApp.Get_DataBase;
      case Command of
        0: CreateClient;
        1: CreateOrder;
      end;
      FDataBase := nil;
      FMainFormHandle := 0;
    except
      on E: Exception do
        MessageBox(ExaApp.Get_MainFormHandle, PChar(E.Message),
          PChar(Get_Caption(Command)), MB_OK);
    end;
  end;
end;

function TPlugin.Get_Caption(Command: Integer): WideString;
begin
  if Command = 0 then
    Result := ' '
  else
    Result := ' ';
end;

function TPlugin.Get_CommandCount: Integer;
begin
  Result := 2;
end;

{ mixed }

function Init: IInterface; safecall;
begin
  Result := TPlugin.Create;
end;

exports
  Init;

end.
